@@ -198,10 +198,16 @@ class Dataset(DataLoader):
         if hasattr(self, "preprocessing_pipeline"):
             for func_name, kwargs in self.preprocessing_pipeline.items():
                 for setting_name, _ in kwargs.items():
+                    if setting_name not in curve_sets and setting_name not in list(
+                        curves
+                    ):
+                        # Avoid non curve related settings
+                        continue
                     # No default for getattr. At this point if the attribute
                     # doesn't exist an error should be raised
-                    new_setting = getattr(self, setting_name)
-                    self.preprocessing_pipeline[func_name][setting_name] = new_setting
+                    self.preprocessing_pipeline[func_name][setting_name] = getattr(
+                        self, setting_name
+                    )
 
     def __ingest_init_input(
         self, att_name: str, att_val: Union[str, Dict[str, Any], Path]
