@@ -464,8 +464,10 @@ def calculate_CALI_BS(df: pd.DataFrame) -> pd.DataFrame:
     Returns:
         pd.DataFrame: Returns the dataframe with CALI-BS as a new column
     """
+    drop_BS = False
     if "CALI" in df.columns:
         if "BS" not in df.columns:
+            drop_BS = True
             df = guess_BS_from_CALI(df)
         df["CALI-BS"] = df["CALI"] - df["BS"]
     else:
@@ -473,6 +475,9 @@ def calculate_CALI_BS(df: pd.DataFrame) -> pd.DataFrame:
             "Not possible to generate CALI-BS. At least CALI needs to be present in the dataset."
         )
 
+    if drop_BS:
+        df = df.drop(columns=["BS"])
+
     return df
 
 
@@ -492,7 +497,7 @@ def calculate_AI(df: pd.DataFrame) -> pd.DataFrame:
         pd.DataFrame: Returns the dataframe with AI as a new column
     """
     if set(["DEN", "AC"]).issubset(set(df.columns)):
-        df.loc[:, "AI"] = df["DEN"] * ((304.8 / df["AC"]) ** 2)
+        df["AI"] = df["DEN"] * ((304.8 / df["AC"]) ** 2)
     else:
         raise ValueError(
             "Not possible to generate AI as DEN and AC are not present in the dataset."
@@ -527,6 +532,7 @@ def calculate_LI(df: pd.DataFrame) -> pd.DataFrame:
             "Not possible to generate LI as NEU and DEN or LFI are not present in dataset."
         )
     df["LI"] = abs(abs(df["LFI"]) - df["LFI"]) / 2
+    df = df.drop(columns=["LFI"])
     return df
 
 
@@ -557,6 +563,7 @@ def calculate_FI(df: pd.DataFrame) -> pd.DataFrame:
             "Not possible to generate FI as NEU and DEN or LFI are not present in dataset."
         )
     df["FI"] = (df["LFI"].abs() + df["LFI"]) / 2
+    df = df.drop(columns=["LFI"])
     return df
 
 
@@ -666,10 +673,12 @@ def calculate_PR(df: pd.DataFrame) -> pd.DataFrame:
     Returns:
         pd.DataFrame: Returns the dataframe with PR as a new column
     """
+    drop = False
     if not set(["VP", "VS"]).issubset(set(df.columns)):
         if set(["AC", "ACS"]).issubset(set(df.columns)):
             df = calculate_VP(df)
             df = calculate_VS(df)
+            drop = True  # Don't want to add unwanted columns
         else:
             raise ValueError(
                 "Not possible to generate PR as none of the neccessary curves "
@@ -678,6 +687,8 @@ def calculate_PR(df: pd.DataFrame) -> pd.DataFrame:
     df["PR"] = (df["VP"] ** 2 - 2.0 * df["VS"] ** 2) / (
         2.0 * (df["VP"] ** 2 - df["VS"] ** 2)
     )
+    if drop:
+        df = df.drop(columns=["VP", "VS"])
     return df
 
 
@@ -840,6 +851,7 @@ def calculate_VSH(df: pd.DataFrame, **kwargs) -> pd.DataFrame:  # noqa: C901
             "DUNLIN GP",
             "BRENT GP",
             "FLADEN GP",
+            "DRAUPNE PGP",  # Draupne pseudogroup
         ],
     }
     mapping_dict = {val: key for key, lst in system_dict.items() for val in lst}
@@ -850,17 +862,21 @@ def calculate_VSH(df: pd.DataFrame, **kwargs) -> pd.DataFrame:  # noqa: C901
         old_index = df.pop("index")
         # Ensure only using legal GR values
         df.loc[df["GR"] < 0, "GR"] = np.nan
-        # Calculate GR_ss
+        # Need to handle Draupne formations separately because of hot shale spots
+        # Generate Draupne pseudo group
+        mask_draupne = df[f_col].str.contains("DRAUPNE").astype("boolean") & df[
+            g_col
+        ].str.contains("VIKING").astype("boolean")
+        df.loc[mask_draupne, g_col] = "DRAUPNE PGP"
+        # Map systems
         df["Age"] = df[g_col].map(mapping_dict)
         # Where not defined in mapping_dict consider as other
         df.loc[df["Age"].isna(), "Age"] = "Other"
-        system_quantiles = df.groupby("Age")["GR"].quantile(quant_ss)
-        df["GR_ss"] = df["Age"].map(system_quantiles)
         # Need to ensure index is continuous after the groupby to prevent
         # different sections of the well being taken into account for the
         # calculation of quant_ss
         system_quantiles = {}
-        for age, age_series in df.groupby("Age")["GR"]:
+        for age, age_series in df.groupby("Age", sort=False)["GR"]:
             # Ensure groups are continuous in index
             sub_ages = age_series.groupby(
                 age_series.index.to_series().diff().ne(1).cumsum()
@@ -869,7 +885,34 @@ def calculate_VSH(df: pd.DataFrame, **kwargs) -> pd.DataFrame:  # noqa: C901
                 sub_age_name = f"{age}_{i}"
                 df.loc[sub_age_series.index, "AGE_GROUPS"] = sub_age_name
                 system_quantiles[sub_age_name] = sub_age_series.quantile(quant_ss)
+
+        # Calculate GR_ss
         df["GR_ss"] = df["AGE_GROUPS"].map(system_quantiles)
+        # Deal with case where mask_jura == mask_draupne, this will drag GR_ss unnecessarily high
+        mask_jura = df["Age"] == "Jurassic"
+        if (mask_jura == mask_draupne).all() and mask_jura.any() and mask_draupne.any():
+            # Make Jurassic system equal to average of system above and below
+            # For now pandas unique is sorted in the order of the values, so we
+            # can just take the value above and below Jurassic
+            keys = df["Age"].unique()
+            idx = np.flatnonzero(keys == "Jurassic")
+            try:
+                system_above = keys[idx + 1][0]
+                closest_subsystem = [
+                    i for i in list(system_quantiles) if system_above in i
+                ][-1]
+                above = system_quantiles[closest_subsystem]
+            except IndexError:
+                above = np.nan
+            try:
+                system_below = keys[idx - 1][0]
+                closest_subsystem = [
+                    i for i in list(system_quantiles) if system_below in i
+                ][-1]
+                below = system_quantiles[closest_subsystem]
+            except IndexError:
+                below = np.nan
+            df.loc[mask_jura, "GR_ss"] = np.nanmean([above, below])
 
         # Need to handle Nordland, Hordaland & Rogaland separetely (see docstring)
         nhr_quant_ss = df.loc[
@@ -897,18 +940,6 @@ def calculate_VSH(df: pd.DataFrame, **kwargs) -> pd.DataFrame:  # noqa: C901
                 if not np.isnan(hord_quant_ss):
                     df.loc[df["Age"] == "Rogaland", "GR_ss"] = hord_quant_ss
 
-        # Also need to handle draupne pseudo group separately
-        # Make all groups in draupne formation have same GR_ss value from
-        # the last group in the draupne formation
-        if f_col is not None:
-            mask = df[f_col].astype(str).str.contains("DRAUPNE").astype("boolean")
-            groups_in_draupne = df.loc[mask, g_col].unique()
-            if mask.sum() > 0 and len(groups_in_draupne) > 1:
-                last_group = groups_in_draupne[-1]
-                df.loc[mask, "GR_ss"] = df.loc[
-                    df[g_col] == last_group, "GR_ss"
-                ].unique()[0]
-
         # Calculate GR_sh_Gp_f
         for group_name, group_series in df.groupby(g_col, dropna=False, sort=False)[
             "GR"
@@ -1040,8 +1071,8 @@ def calculate_VSH(df: pd.DataFrame, **kwargs) -> pd.DataFrame:  # noqa: C901
                             inter_start = group_start
                             inter_end = np.max(inter_end)
                         elif end_na.any():
-                            inter_end = group_end
                             vertical = df.loc[inter_start, "GR_sh_Gp_f"].idxmax()
+                            inter_end = group_end
                             inter_start = np.min(inter_start)
                         else:
                             vertical = df.loc[
@@ -1050,19 +1081,31 @@ def calculate_VSH(df: pd.DataFrame, **kwargs) -> pd.DataFrame:  # noqa: C901
                             inter_start = np.min(inter_start)
                             inter_end = np.max(inter_end)
                     else:
+                        # Threshold violation is within a group then just
+                        # apply normal sign_change algorithm
                         inter_start, inter_end = utilities.inflection_points(
                             df, "GR_sh_Gp_f", start, end
                         )
 
-                    # If only one inflection point, draw a vertical line
-                    inter_mask = np.isnan([inter_start, inter_end])
-                    if inter_mask.any():
-                        if np.where(inter_mask)[0][0] == 0:
-                            inter_start = group_start
-                            vertical = inter_end
-                        else:
-                            inter_end = group_end
-                            vertical = inter_start
+                        # If only one inflection point, draw a vertical line
+                        inter_mask = np.isnan([inter_start, inter_end])
+                        if inter_mask.any():
+                            if np.where(inter_mask)[0][0] == 0:
+                                inter_start = group_start
+                                vertical = inter_end
+                            else:
+                                inter_end = group_end
+                                vertical = inter_start
+
+                    # Regardless of above situation dont allow interpolation
+                    # beyond the scope of the group since the threshold violation
+                    # only applies to the group.
+                    inter_start = max(inter_start, group_start - 1)
+                    inter_end = min(inter_end, group_end + 1)
+
+                    # TODO: Evaluate how close inter_start/inter_end are to the
+                    #     start/end of the group. If they are close then just
+                    #     interpolate between the start and end of the group.
                 else:
                     # Threshold violation spans multiple groups. Just find
                     # start and end points for interpolation based purely
@@ -1082,10 +1125,8 @@ def calculate_VSH(df: pd.DataFrame, **kwargs) -> pd.DataFrame:  # noqa: C901
                             vertical = inter_start
 
                 # Bound indices to not go beyond the scope of the dataframe
-                if inter_start < df.index[0]:
-                    inter_start = df.index[0]
-                if inter_end > df.index[-1]:
-                    inter_end = df.index[-1]
+                inter_start = max(inter_start, df.index[0])
+                inter_end = min(inter_end, df.index[-1])
 
                 new_GR_sh_Gp_f = df.loc[inter_start:inter_end, "GR_sh_Gp_f_NS"].copy()
                 # If vertical is not nan, it means we need to draw a vertical line
@@ -1097,7 +1138,7 @@ def calculate_VSH(df: pd.DataFrame, **kwargs) -> pd.DataFrame:  # noqa: C901
                     vertical = np.clip(vertical, inter_start, inter_end)
                     new_GR_sh_Gp_f.loc[:] = new_GR_sh_Gp_f.loc[vertical]
 
-                # Last check, if the new NS curve is less than the original
+                # last check, if the new NS curve is less than the original
                 # filtered rolling window curve then discard the interpolation
                 original = df.loc[inter_start:inter_end, "GR_sh_Gp_f_NS"]
                 check_mask = new_GR_sh_Gp_f < original
@@ -1127,8 +1168,10 @@ def calculate_VSH(df: pd.DataFrame, **kwargs) -> pd.DataFrame:  # noqa: C901
         else:
             df["VSH"] = (df["GR"] - df["GR_ss"]) / (df["GR_sh_Gp_f"] - df["GR_ss"])
         df["VSH"] = df["VSH"].clip(lower=0, upper=1)
+
         # And remap the old index to restore df to original state
         df.index = df.index.map(old_index)
+        df = df.sort_index()
 
         # Drop unused columns
         df = df.drop(
@@ -1161,6 +1204,7 @@ def calculate_VSH(df: pd.DataFrame, **kwargs) -> pd.DataFrame:  # noqa: C901
     df[g_col] = utilities.map_formation_and_group(
         df[g_col].apply(utilities.standardize_group_formation_name)
     )[1]
+    df[g_col] = df[g_col].astype(str)
 
     # If no formations_columns_name was provided, warn the user but continue
     if f_col is None:
@@ -1175,6 +1219,7 @@ def calculate_VSH(df: pd.DataFrame, **kwargs) -> pd.DataFrame:  # noqa: C901
         df[f_col] = utilities.map_formation_and_group(
             df[f_col].apply(utilities.standardize_group_formation_name)
         )[0]
+        df[f_col] = df[f_col].astype(str)
 
     # Process per well
     well_names = df[id_column].unique()
