@@ -1,25 +1,25 @@
 # ONLY testing one function from feature_engineering. Would be good to expand this
 # if we have some spare time in a sprint
-import pytest
-from pandas.testing import assert_frame_equal
-import pandas as pd
 import numpy as np
+import pandas as pd
+import pytest
+from cognite.client import CogniteClient
+from pandas.testing import assert_frame_equal
 
 from akerbp.mlpet import feature_engineering
 from akerbp.mlpet.tests.data.data import (
     FORMATION_DF,
     FORMATION_TOPS_MAPPER,
-    VERTICAL_DF,
-    VERTICAL_DEPTHS_MAPPER,
     TEST_DF,
+    VERTICAL_DEPTHS_MAPPER,
+    VERTICAL_DF,
 )
-from cognite.client import CogniteClient
 
 client = CogniteClient(client_name="test", project="akbp-subsurface")
 
 
-def test_add_formation_tops_using_mapper():
-    df_with_tops = feature_engineering.add_formation_tops_label(
+def test_add_formations_and_groups_using_mapper():
+    df_with_tops = feature_engineering.add_formations_and_groups(
         FORMATION_DF[["DEPTH", "well_name"]],
         formation_tops_mapper=FORMATION_TOPS_MAPPER,
         id_column="well_name",
@@ -28,8 +28,8 @@ def test_add_formation_tops_using_mapper():
     assert_frame_equal(df_with_tops.sort_index(axis=1), FORMATION_DF.sort_index(axis=1))
 
 
-def test_add_formation_tops_using_client():
-    df_with_tops = feature_engineering.add_formation_tops_label(
+def test_add_formations_and_groups_using_client():
+    df_with_tops = feature_engineering.add_formations_and_groups(
         FORMATION_DF[["DEPTH", "well_name"]],
         id_column="well_name",
         client=client,
@@ -37,12 +37,12 @@ def test_add_formation_tops_using_client():
     assert_frame_equal(df_with_tops.sort_index(axis=1), FORMATION_DF.sort_index(axis=1))
 
 
-def test_add_formation_tops_raises_exception_if_no_client_nor_mapping_is_provided():
+def test_add_formations_and_groups_raises_exception_if_no_client_nor_mapping_is_provided():
     with pytest.raises(
         Exception,
         match="Neither a formation tops mapping nor cognite client is provided. Not able to add formation tops to dataset",
     ):
-        _ = feature_engineering.add_formation_tops_label(
+        _ = feature_engineering.add_formations_and_groups(
             FORMATION_DF[["DEPTH", "well_name"]],
             id_column="well_name",
         )
@@ -89,7 +89,7 @@ def test_add_vertical_depths_raises_expection_if_no_client_nor_mapping_is_provid
 def test_add_well_metadata():
     metadata = {"30/11-6 S": {"FOO": 0}, "25/7-4 S": {"FOO": 1}}
     df = feature_engineering.add_well_metadata(
-        TEST_DF, metadata_dict=metadata, columns=["FOO"], id_column="well_name"
+        TEST_DF, metadata_dict=metadata, metadata_columns=["FOO"], id_column="well_name"
     )
     assert "FOO" in df.columns.tolist()
 
@@ -112,7 +112,10 @@ def test_calculate_VSH():
     )
     df["well"] = "dummy"
     df_out = feature_engineering.calculate_VSH(
-        df, formation_groups_column_name="GROUP", id_column="dummy"
+        df,
+        groups_column_name="GROUP",
+        formations_column_name="FORMATION",
+        id_column="well",
     )
     assert "VSH" in df_out.columns.tolist()
     assert not df_out["VSH"].isnull().values.any()
