@@ -1069,10 +1069,13 @@ def inflection_points(
     for series_name, series in inflection_points.items():
         try:
             with np.errstate(invalid="ignore"):
+                first_derivative = np.gradient(series)
+                if np.isnan(first_derivative).all():
+                    raise IndexError
                 first_inflection_point = (
                     np.where(
                         np.diff(
-                            pd.Series(np.sign(np.gradient(series)))
+                            pd.Series(np.sign(first_derivative))
                             .replace(0, np.nan)
                             .fillna(method="ffill")
                         )
@@ -1082,11 +1085,13 @@ def inflection_points(
                 )[0]
                 first_inflection_point = series.index[first_inflection_point]
         except ValueError:
-            # Not enough data points to take a gradient
-            # Just return the only index in the series
+            # If ValueError, not enough data points to take a gradient, i.e. only
+            # one point in the series. In this case, the first inflection point
+            # is the first and only point in the series.
             first_inflection_point = series.index[0]
         except IndexError:
-            # No inflection point in provided series
+            # If IndexError, no inflection point found
+            # In both cases return np.nan
             first_inflection_point = np.nan
         inflection_points[series_name] = first_inflection_point
 
